/**
 * @PropertiesManager.java 24.11.2010 build: 24.11.2010 Copyright 2010
 *                         yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 *         This file is a part of Sudoku Sudoku is free software; you can
 *         redistribute it and/or modify it under the terms of the GNU General
 *         Public License as published by the Free Software Foundation; either
 *         version 2 of the License, or (at your option) any later version.
 * 
 *         Sudoku is distributed in the hope that it will be useful, but WITHOUT
 *         ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *         FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *         for more details.
 * 
 *         You should have received a copy of the GNU General Public License
 *         along with Sudoku; if not, write to the Free Software Foundation,
 *         Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
package de.yourwebs.sudoku;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Enumeration;
import java.util.Properties;

/**
 * The class handles the options settings.
 */
public class PropertiesManager {
    /**
     * The config file's absolute path
     */
    private final String configFilePath;

    /**
     * The configuration file's header
     */
    private final String header = "# Yourwebs.de - Sudoku\n"
            + "Sudoku Configuration File";

    /**
     * The default properties are saved when the properties file is created. The
     * properties list is preset by {@link de.yourwebs.sudoku.SudokuConstants
     * SudokuConstants} and may be changed by the game's options dialog and menu
     * bar.
     * 
     * @see de.yourwebs.sudoku.PropertiesManager#properties
     */
    private final Properties defaultProperties = new Properties();

    /**
     * A variable list of settings defined by the game's options dialog and menu
     * bar. By default {@link #defaultProperties} are loaded.
     */
    private Properties properties = new Properties();

    /**
     * Construcs the default properties of the game. The default values are
     * initialized by {@link de.yourwebs.sudoku.SudokuConstants SudokuConstants}
     * 
     * @see #defaultProperties
     * @see #properties
     */
    public PropertiesManager() {

        configFilePath = Config.CONFIG_DIR.value
                + System.getProperty("file.separator") + "config";

        defaultProperties.put("sizeSlider", SudokuConstants.GUI_SIZE);
        defaultProperties.put("fontWeight", SudokuConstants.FONT_WEIGHT);
        defaultProperties.put("contrast", SudokuConstants.ACCESSIBILITY);
        defaultProperties.put("mistakeHint", SudokuConstants.MISTAKE_HINT);
        defaultProperties
                .put("isSuccessSoundPlayable", SudokuConstants.PLAY_SUCCESS_SOUND);
        defaultProperties
                .put("isSpeechPlayable", SudokuConstants.PLAY_VOICE_OUTPUT);
        defaultProperties.put("difficulty", SudokuConstants.DIFFICULTY);
        defaultProperties.put("mistakeDelay", SudokuConstants.MISTAKE_DELAY);
    }

    /**
     * Creates the configuration directory and file in the
     * {@linkplain de.yourwebs.sudoku.Config#CONFIG_DIR configuration directory}
     * if it not already exists. The directory is created at the specified path
     * including its superdirectories.
     * 
     * @return <code>true</code> if the configuration directory and file already
     *         exists or is created successfully otherwise <code>false</code>.
     */
    private boolean makeConfigFile() {

        final File file = new File(configFilePath);
        final File dir = file.getParentFile();

        if (!dir.isDirectory() && !dir.mkdirs()) {
            System.err.println(dir
                    + " ist kein Verzeichnis und kann nicht erstellt werden!");
            return false;
        }

        try {
            if (file.isFile() && file.canRead() && file.canWrite()) {
                System.out.println(file
                        + " ist vorhanden und lesbar und beschreibbar.");
                return true;
            }
            if (file.createNewFile()) {
                System.out.println(file + " erstellt.");
                return true;
            }
        } catch (final IOException e) {
            System.err
                    .println("Konfigurationsdatei konnte nicht angelegt werden.");
            e.printStackTrace();
        }
        return false;
    }

    /**
     * The method is called by the game's option dialog and menu bar forwarding
     * to {@link #saveProperties(Properties) saveProperties(Properties)} set its
     * parabeter to <code>null</code>. The properties list usually is defined by
     * the user and read from configuration file.
     */
    public void saveProperties() {
        this.saveProperties(null);
    }

    /**
     * Saves the used {@link #properties} list as XML to {@link #configFilePath}
     * 
     * @param prop
     *            the properties object that has to be saved. If
     *            <code>prop</code> is <code>null</code> all the actually used
     *            {@link #properties properties object} is saved.
     */
    public void saveProperties(Properties prop) {
        prop = prop == null ? properties : prop;

        if (configFilePath == null || configFilePath.length() == 0) {
            return;
        }

        FileOutputStream propOut = null;
        try {
            propOut = new FileOutputStream(configFilePath);
            properties.putAll(prop);
            prop.storeToXML(propOut, header);
        } catch (final FileNotFoundException e) {
            e.printStackTrace();
        } catch (final IOException e) {
            e.printStackTrace();
        } finally {
            try {
                if (propOut != null) {
                    propOut.close();
                }
            } catch (final IOException e) {
            }
        }
    }

    /**
     * Reads the properties from file. If it's not readable or does not exist,
     * the {@link #defaultProperties} are used for the game and saved to a newly
     * created config file.<br>
     * Called when the game's gui is initialized.
     */
    public void loadProperties() {

        if (configFilePath == null || configFilePath.length() == 0) {
            return;
        }

        final File f = new File(configFilePath);
        final Enumeration<?> propEnum = defaultProperties.propertyNames();
        String prop;

        if (f.canRead() && f.isFile() && f.length() > 0) {
            System.out.println(f + " ist lesbar");
            FileInputStream propIn = null;
            try {
                propIn = new FileInputStream(configFilePath);
                properties.loadFromXML(propIn);
                // durchlaeuft die Keys der default Properties und prueft, ob
                // diese in
                // den ausgelesenen Properties belegt sind. Falls nicht wird ein
                // Leerstring gespeichert.
                while (propEnum.hasMoreElements()) {
                    prop = (String) propEnum.nextElement();
                    if (properties.getProperty(prop) == null) {
                        properties.setProperty(prop, "");
                    }
                }
                return;
            } catch (final FileNotFoundException e) {
                e.printStackTrace();
            } catch (final IOException e) {
                e.printStackTrace();
            } finally {
                try {
                    if (propIn != null) {
                        propIn.close();
                    }
                } catch (final IOException e) {
                }
            }
        }
        /* if a config file is not readable or does not exist */
        properties = defaultProperties;
        System.out.println(f
                + " ist nicht lesbar oder nicht vorhaden, versuche sie zu erstellen...");
        if (this.makeConfigFile()) {
            System.out.println(f + " erfolgreich erstellt");
        }
        this.saveProperties(properties);
    }

    /**
     * Adds a key-value-pair to the actual properties. If these are not set the
     * default properties are loaded
     * 
     * @param key
     *            <code>String</code>, the key
     * @param value
     *            <code>String</code>, the value
     */
    public void setProperty(final String key, final String value) {
        if (properties == null) {
            properties = defaultProperties;
        } else {
            properties.setProperty(key, value);
        }
    }

    /**
     * Provides the value of a property's key
     * 
     * @param key
     *            <code>String</code> the key
     * @return <code>String</code>, the value belonging to <code>key</code>.
     */
    public String getProperty(final String key) {
        if (properties == null) {
            properties = defaultProperties;
        }
        return properties.getProperty(key);
    }
}
