/**
 * @Element.java 14.10.2010 Copyright 2010 - 2017 javabeginners.de. All rights
 *               reserved
 * @author J&ouml;rg Czeschla
 *
 *         This file is a part of Sudoku Sudoku is free software; you can
 *         redistribute it and/or modify it under the terms of the GNU General
 *         Public License as published by the Free Software Foundation; either
 *         version 2 of the License, or (at your option) any later version.
 * 
 *         Sudoku is distributed in the hope that it will be useful, but WITHOUT
 *         ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 *         FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *         for more details.
 * 
 *         You should have received a copy of the GNU General Public License
 *         along with Spiele; if not, write to the Free Software Foundation,
 *         Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 */
package de.yourwebs.sudoku.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;

import javax.swing.JPanel;
import javax.swing.border.LineBorder;

import de.yourwebs.sudoku.SudokuConstants;

/**
 * The class provides a <code>JPanel</code> with a <code>JTextField</code> and a
 * <code>JLabel</code> situated over each other. Both text components are
 * editable and can be set transparent while changing their z-layer. Hence a
 * play-mode and a separate hint-mode can be activated.
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
public class Element extends JPanel implements MouseListener {

    /**
     * Index of the square this element belongs to
     */
    private int quad;

    /**
     * Index of the row this element belongs to
     */
    private int row;

    /**
     * Index of the column this element belongs to
     */
    private int col;

    /**
     * Used for scaling the fonts proportionally. The puzzle numbers font-size
     * is scaled by <code>faktor</code>, the hint numbers font-size is scaled by
     * <code>faktor / 2 - 1</code>.
     */
    private int faktor;

    /**
     * <p>
     * Set by the properties file to switch the font-weight between
     * <em>plain</em> and <em>bold</em>.
     * </p>
     * <p>
     * Default: {@link java.awt.Font#PLAIN }
     * </p>
     */
    private int weight = SudokuConstants.FONT_WEIGHT;

    /**
     * <p>
     * The game's main window.
     * </p>
     */
    private final SudokuGUI gui;

    /**
     * <p>
     * The foreground color
     * </p>
     * <p>
     * Default: {@link java.awt.Color#BLACK}
     * </p>
     */
    private Color fgColor = Color.BLACK;

    /**
     * <p>
     * The border color
     * </p>
     * <p>
     * Default: {@link java.awt.Color#BLACK}
     * </p>
     */
    private Color borderColor = Color.BLACK;

    /**
     * <p>
     * The element's border. Its size is proportionally scaled by
     * {@link #faktor}.
     * </p>
     */
    private LineBorder border;

    /**
     * <p>
     * The element's text field used to enter the puzzle number.
     * </p>
     */
    private final NumberTextField field;
    /**
     * <p>
     * The element's hint field used to enter the game's hints.
     * </p>
     */
    private final HintLabel label;

    /**
     * Creates an element panel with a field or label. An element displays
     * either a number from 1 to 9 at the textfield or multiple numbers at the
     * label.
     * 
     * @param gui
     *            The main frame
     * @param faktor
     *            A numeric factor used for proportionally scaling
     * @param fontWeight
     *            Weigt of the used fonts
     */
    public Element(final SudokuGUI gui, final int faktor,
            final int fontWeight) {
        this.gui = gui;
        this.faktor = faktor;
        this.weight = fontWeight;
        this.border = new LineBorder(borderColor, faktor / 16);
        // Number Text Field
        field = new NumberTextField();
        field.setCaretColor(fgColor);
        field.addMouseListener(this);
        field.setMinimumSize(this.getSize());
        field.setPreferredSize(this.getSize());
        field.setSize(new Dimension(this.getBounds().width,
                this.getBounds().height));
        // Input Eventhandling
        field.addKeyListener(new KeyAdapter() {
            @Override
            public void keyReleased(final KeyEvent e) {
                gui.handleInput(Element.this);
                // Delete hints when entering a number
                if (!label.getText().equals("")
                        && field.getText().matches("\\d+")) {
                    label.setText("");
                }
            }

            @Override
            public void keyPressed(final KeyEvent e) {
                // Navigation by arrow keys.
                // If called in keyReleased() highlighting
                // of mistakes is disabled.
                gui.walkCursor(Element.this, e.getKeyCode());
            }
        });
        // Hint Label
        final Dimension dim = new Dimension(100, 100);
        label = new HintLabel("<html> </html>");
        label.setMinimumSize(dim);
        label.setPreferredSize(dim);
        label.setSize(dim);
        label.addMouseListener(this);
        // Layout
        this.setLayout(new GridBagLayout());
        final GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.insets = new Insets(0, 0, 0, 0);
        gbc.weightx = 1;
        gbc.weighty = 1;
        this.add(field, gbc, 1, 1, 1, 1, this);
        this.add(label, gbc, 1, 1, 1, 1, this);
    }

    @Override
    public void paintComponent(final Graphics g) {
        super.paintComponent(g);
        this.setBorder(border);
    }

    /**
     * Used for navigation by arrow keys. The method passes the focus from the
     * panel to the textfield
     */
    @Override
    public void requestFocus() {
        field.requestFocus();
    }

    /**
     * Factory-method to place a component onto the <code>JPanel p</code> layed
     * out in a <code>GridBagLayout</code>.
     * 
     * @param c
     *            <code>Component</code>, die Komponente, die gesetzt wird
     * @param constraints
     *            <code>GridBagConstraints</code>, die Constraints
     * @param x
     *            <code>int</code>, X-Wert der linken oberen Ecke in rel.
     *            Einheiten
     * @param y
     *            <code>int</code>, Y-Wert der linken oberen Ecke in rel.
     *            Einheiten
     * @param w
     *            <code>int</code>, Breite in rel. Einheiten
     * @param h
     *            <code>int</code>, H&ouml;he in rel. Einheiten
     * @param p
     *            <code>JPanel</code>, the panel with <code>GridBagLayout</code>
     *            to place the component <code>c</code>.
     */
    private void add(final Component c, final GridBagConstraints constraints,
            final int x, final int y, final int w, final int h,
            final JPanel p) {
        constraints.gridx = x;
        constraints.gridy = y;
        constraints.gridwidth = w;
        constraints.gridheight = h;
        p.add(c, constraints);
    }

    /**
     * Removes the numbers of the hint label, setting a HTML-Element with a
     * space (<code>&lt;html&gt;&nbsp;&lt;/html&gt;</code>) as lable text.
     */
    public void clearHintLabelText() {
        label.setText("<html> </html>");
    }

    /**
     * <p>
     * Sets the puzzle's number <code>txt</code> on the text field. Used when
     * generating a new puzzle, reloading a game, help or showing mistakes.
     * </p>
     * 
     * @param txt
     *            <code>String</code> the puzzle's number.
     */
    public void setFieldText(final String txt) {
        field.setText(txt);
    }

    /**
     * 
     * @return <code>String</code> the puzzle's number shown in a filled out
     *         sudoku cell
     */
    public String getFieldText() {
        return field.getText();
    }

    /**
     * 
     * @return <code>HintLabel</code> the label that shows the hint numbers
     */
    public HintLabel getLabel() {
        return label;
    }

    /**
     * 
     * @return <code>NumberTextField</code> the text field that shows the
     *         puzzle's number
     */
    public NumberTextField getField() {
        return field;
    }

    /**
     * Sets the color of a selected field indidating either a mistake showing
     * two or more identical numbers in a row, column or square, or all similar
     * numbers in the whole game.
     * 
     * @param c
     *            <code>Color</code> the color to be set.
     */
    public void setSelectionColor(final Color c) {
        field.setSelectionColor(c);
    }

    /**
     * Sets the foreground color to the puzzle text field and to the hint label.
     * 
     * @param fieldFG
     *            <code>Color</code> foreground color for the puzzle text field.
     * @param labelFG
     *            <code>Color</code> foreground color for the hint text label.
     */
    public void setForeground(final Color fieldFG, final Color labelFG) {
        if (field != null) {
            field.setForeground(fieldFG);
            field.repaint();
        }

        if (label != null) {
            label.setForeground(labelFG);
            label.repaint();
        }
        fgColor = fieldFG;
    }

    /**
     * Sets the background color to the puzzle text field and to the hint label.
     * 
     * @param fieldBG
     *            <code>Color</code> background color for the puzzle text field.
     * @param labelBG
     *            <code>Color</code> background color for the hint text label.
     */
    public void setBackground(final Color fieldBG, final Color labelBG) {
        if (field != null) {
            field.setBackground(fieldBG);
            field.repaint();
        }

        if (label != null) {
            label.setBackground(labelBG);
            label.repaint();
        }
    }

    /**
     * Sets the font-weight of the colors.
     * 
     * @param weight
     *            <code>int</code>, the font-weight either
     *            {@link java.awt.Font#PLAIN} or {@link java.awt.Font#BOLD}.
     */
    public void setFontWeight(final int weight) {
        this.weight = weight;
    }

    /**
     * Reloads the fonts for the puzzle field and the hint label. Both faces are
     * defined by {@link de.yourwebs.sudoku.SudokuConstants#FONT_FACE}. The
     * font-weight may be switched from {@link java.awt.Font#PLAIN} to
     * {@link java.awt.Font#BOLD} by the values given in the options panel and
     * the font-size is calculated depending on {@link #faktor} and some hard
     * coded settings.
     */
    public void reloadFont() {
        field.setFont(new Font(SudokuConstants.FONT_FACE, weight, this.faktor));
        field.repaint();
        label.setFont(new Font(SudokuConstants.FONT_FACE, weight,
                this.faktor / 2 - 1));
        label.repaint();
        this.revalidate();
    }

    /**
     * 
     * @return The square index
     */
    public int getQuad() {
        return quad;
    }

    /**
     * Sets the square index of this wlement
     * 
     * @param quad
     *            The index of this ekenebt's square
     */
    public void setQuad(final int quad) {
        this.quad = quad;
    }

    /**
     * 
     * @return The row index
     */
    public int getRow() {
        return row;
    }

    /**
     * Sets the row index of this wlement
     * 
     * @param row
     *            The index of this element's row
     */
    public void setRow(final int row) {
        this.row = row;
    }

    /**
     * 
     * @return The column index
     */
    public int getCol() {
        return col;
    }

    /**
     * Sets the column index of this wlement
     * 
     * @param col
     *            The index of this element's column
     */
    public void setCol(final int col) {
        this.col = col;
    }

    /**
     * Sets the rescaling factor
     * 
     * @param faktor
     *            The rescaling factor's value
     */
    public void setFaktor(final int faktor) {
        this.faktor = faktor;
    }

    /**
     * Sets the element's foreground color
     * 
     * @param color
     *            The foreground color
     */
    public void setFGColor(final Color color) {
        fgColor = color;
        field.repaint();
    }

    /**
     * Sets the element's border color
     * 
     * @param color
     *            The foreground color
     */
    public void setBorderColor(final Color color) {
        borderColor = color;
        border = new LineBorder(borderColor, faktor / 16);
        this.repaint();
    }

    @Override
    public void mouseClicked(final MouseEvent e) {
    }

    @Override
    public void mousePressed(final MouseEvent e) {
    }

    @Override
    public void mouseReleased(final MouseEvent e) {
        gui.highlightNumber(field.getText());
    }

    @Override
    public void mouseEntered(final MouseEvent e) {
    }

    @Override
    public void mouseExited(final MouseEvent e) {
    }
}
