package de.yourwebs.sudoku.gui;

import java.awt.Color;
import java.awt.Font;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionAdapter;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.border.LineBorder;

/**
 * An editable <code>JLabel</code> used to show some hint numbers the player may
 * use for remembrance.
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
public class HintLabel extends JLabel {

    /*-------------- Color-, Border- and Font-Constants --------------*/

    /**
     * The label's background color
     */
    public static final Color BG_COLOR = new Color(0.7f, 0.7f, 0.9f, 1f);
    /**
     * Color of the hint label's border shown when the label has focus
     */
    private static final Color FOCUS_BORDER_COLOR = Color.ORANGE;
    /**
     * Color of the hint label's border shown when hint mode is active
     */
    private static final Color ACTIVE_BORDER_COLOR = new Color(1f, 0.5f, 0f,
            1f);
    /**
     * Border of the hint label shown when the label has focus
     */
    static final Border FOCUS_BORDER = new LineBorder(FOCUS_BORDER_COLOR, 3);
    /**
     * Border of the hint label shown when hint mode is active
     */
    public static final Border ACTIVE_BORDER = new LineBorder(
            ACTIVE_BORDER_COLOR, 3);
    /**
     * The label's font
     */
    private static final Font LABEL_FONT = new Font("Verdana", Font.PLAIN, 15);

    /**
     * The focus state
     */
    private boolean focus = false;

    /**
     * Constructs a hint label with the given text and a center alignment
     * 
     * @param txt
     *            The text displayed on the label
     */
    public HintLabel(final String txt) {
        super(txt, SwingConstants.CENTER);
        this.setFont(LABEL_FONT);
        this.setOpaque(true);
        this.setBackground(BG_COLOR);

        /****************** Event Handling ***************/

        // set and remove a border when gaining or losing the focus
        this.addFocusListener(new HintFocusListener(this));

        // gain the focus if mouse is pressed
        this.addMouseListener(new HintMouseListener(this));

        // set a tooltip when moving the mouse over a non empty label
        this.addMouseMotionListener(new HintMouseMotionListener(this));

        // read and process the hints
        this.addKeyListener(new HintKeyListener(this));
    } // Konstruktor

    /**
     * 
     * @return The focus state
     * @see #focus
     */
    boolean getFocus() {
        return focus;
    }

    /**
     * Sets the focus state
     * 
     * @param focus
     *            The focus state
     * @see #focus
     */
    void setFocus(final boolean focus) {
        this.focus = focus;
    }

    /**
     * Processes the label's text and displays a bunch of unique numbers between
     * 1 and 9. The numbers are displayed in 1 or 2 rows. &lt;Space&gt; deletes
     * the entry.
     * 
     * @param txt
     *            <code>String</code> the text (number) to be appended to the
     *            given label text.
     * @return the processed HTML-String that will be set to the label.
     */
    String processLabelString(final String txt) {
        // blank clears the label
        if (txt.equals(" ")) {
            return txt;
        }
        // get the label's text
        String str = this.getText();
        // ignore the number if it already exists
        if (str.contains(txt)) {
            return this.deleteNumber(str, txt);
        }
        // delete html tags
        str = str.replaceAll("<[^>]*>", "").trim();
        str += txt;
        final char[] chars = str.toCharArray();
        Arrays.sort(chars);

        final ArrayList<Character> list = new ArrayList<Character>();

        for (final char c : chars) {
            if (list.contains(c)) {
                continue;
            }
            list.add(c);
        }
        final Object[] obj = list.toArray();
        final StringBuffer buff = new StringBuffer("<html>");
        for (int i = 0; i < obj.length; ++i) {
            if (i != 0 && i % 4 == 0) {
                buff.append("<br>" + obj[i]);
            } else {
                buff.append(obj[i]);
            }
        }
        buff.append("</html>");
        return buff.toString();
    }

    /**
     * Removes <code>txt</code> from <code>str</code> if <code>txt</code> exists
     * once in <code>str</code>.
     * 
     * @param str
     *            the <code>String</code> to be examined
     * @param txt
     *            the <code>String</code> to be searched for
     * @return the processed <code>String</code>
     */
    private String deleteNumber(final String str, final String txt) {
        final String partOne = str.substring(0, str.indexOf(txt));
        final String partTwo = str
                .substring(str.indexOf(txt) + 1, str.length());
        return partOne + partTwo;
    }
}

/**
 * The hint label's FocusListener
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
class HintFocusListener implements FocusListener {
    /**
     * The label firing the focus event
     */
    private final HintLabel hl;

    /**
     * Listenes to the focus event initialized by the given label
     * 
     * @param hl
     *            The label firing the focus event
     */
    public HintFocusListener(final HintLabel hl) {
        this.hl = hl;
    }

    @Override
    public void focusGained(final FocusEvent e) {
        // set an orange border on focus
        hl.setFocus(true);
        hl.setBorder(HintLabel.FOCUS_BORDER);
        hl.repaint();
    }

    @Override
    public void focusLost(final FocusEvent e) {
        // change the border if focus is lost
        hl.setFocus(false);
        hl.setBorder(HintLabel.ACTIVE_BORDER);
        hl.repaint();
    }
}

/**
 * The hint label's MouseListener
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
class HintMouseListener extends MouseAdapter {
    /**
     * The label firing the mouse event
     */
    private final HintLabel hl;

    /**
     * Listenes to mouse events initialized by the given label
     * 
     * @param hl
     *            The label firing the mouse event
     */
    public HintMouseListener(final HintLabel hl) {
        this.hl = hl;
    }

    @Override
    public void mousePressed(final MouseEvent e) {
        hl.setFocus(true);
        hl.requestFocus(hl.getFocus());
    }
}

/**
 * The hint label's MouseMotionListener
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
class HintMouseMotionListener extends MouseMotionAdapter {
    /**
     * The label firing a mouse motion event
     */
    private final HintLabel hl;

    /**
     * Listenes to mouse motion events initialized by the given label
     * 
     * @param hl
     *            The label firing the mouse motion event
     */
    public HintMouseMotionListener(final HintLabel hl) {
        this.hl = hl;
    }

    @Override
    public void mouseMoved(final MouseEvent e) {
        final String txt = hl.getText();
        if (txt.matches("<html>[1-9]*</html>")) {
            hl.setToolTipText(txt);
        }
    }
}

/**
 * The hint label's KeyListener
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
class HintKeyListener extends KeyAdapter {
    /**
     * The label firing a key event
     */
    private final HintLabel hl;

    /**
     * Listenes to key events initialized by the given label
     * 
     * @param hl
     *            The label firing the key event
     */
    public HintKeyListener(final HintLabel hl) {
        this.hl = hl;
    }

    @Override
    public void keyReleased(final KeyEvent e) {
        String txt = "";
        final int num = e.getKeyCode();
        if (num > 48 && num < 58) { // allow only numbers to be
                                    // displayed
            txt = Character.valueOf((char) num).toString();
        } else if (num == 32) { // clear entry by space
            txt = " ";
        } else {
            return;
        } // don't allow any others
        txt = hl.processLabelString(txt);
        hl.setText(txt);
        hl.setToolTipText(txt);
        hl.repaint();
        hl.revalidate();
    }
}
