/*
 * @OptionDialog.java 20.11.2010 Copyright 2010 - 2013 javabeginners.de. All
 * rights reserved
 * 
 * @author J&ouml;rg Czeschla
 *
 * Einstellungs-Dialog des Sudokus
 *
 * This file is a part of Sudoku Sudoku is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * Sudoku is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Sudoku; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA 02111-1307, USA.
 */
package de.yourwebs.sudoku.gui;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.text.NumberFormat;

import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.NumberFormatter;

/**
 * A dialog window to adjust the game's settings
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
public class OptionDialog extends JDialog
        implements ChangeListener, CaretListener {

    /**
     * A slider to adjust the size of the game
     */
    JSlider slider;
    /**
     * A checkbox to switsch between bold and normal font. If selected a bold
     * font face is shown
     */
    JCheckBox boldBox;
    /**
     * A checkbox to switsch between high contrast and normal contrast mode. If
     * selected the game is displayed in high contrast
     */
    JCheckBox contrastBox;
    /**
     * If the checkbox is selected, the row, column and square in which a
     * mistake is made is highlighted.
     */
    JCheckBox mistakeBox;
    /**
     * If the checkbox is selected, a sound is played when a game is finished
     * successfully.
     */
    JCheckBox successSoundBox;
    /**
     * If the checkbox is selected, a spoken sound is played when a number is
     * entered
     */
    JCheckBox speechBox;
    /**
     * A text field to enter the milliseconds a mistake hint is shown if
     * enabled.
     */
    JFormattedTextField delayTextField;
    /**
     * The main frame
     */
    SudokuGUI mainGUI;

    /**
     * Constucts a dialog used to set the game's options
     * 
     * @param gui
     *            Owner of the dialog
     */
    public OptionDialog(final SudokuGUI gui) {
        this.mainGUI = gui;
        this.initGUI();
        this.setLocationRelativeTo(mainGUI);
        this.setTitle("Einstellungen");
        this.pack();
        this.setVisible(true);
    }

    /**
     * Creates and initializes the dialog's gui
     */
    private void initGUI() {
        final JLabel sizeLabel = new JLabel("Gr\u00F6\u00DFe:");
        final int min = 175;
        final int max = 500;

        final JLabel fettLabel = new JLabel("Fettschrift:");
        int f = mainGUI.getFaktor();
        f = f >= min ? f : min;
        f = f <= max ? f : max;
        slider = new JSlider(min, max, f);
        slider.addChangeListener(this);

        boldBox = new JCheckBox();
        boldBox.setSelected(mainGUI.getFontWeight() == 1 ? true : false);
        boldBox.addChangeListener(this);

        final JLabel accessLabel = new JLabel("Kontrast:");

        contrastBox = new JCheckBox();
        contrastBox.setSelected(mainGUI.getContrast());
        contrastBox.addChangeListener(this);

        final JLabel mistakeLabel = new JLabel("Fehlerhinweis");

        mistakeBox = new JCheckBox();
        mistakeBox.setSelected(mainGUI.getMistakeHint());
        mistakeBox.addChangeListener(this);

        final JLabel successSoundLabel = new JLabel(
                "Akustische Erfolgsmeldung");

        successSoundBox = new JCheckBox();
        successSoundBox.setSelected(mainGUI.getSuccessSoundPlayable());
        successSoundBox.addChangeListener(this);

        final JLabel speechLabel = new JLabel("Sprachausgabe");

        speechBox = new JCheckBox();
        speechBox.setSelected(mainGUI.getSpeechPlayable());
        speechBox.addChangeListener(this);

        final JLabel delayLabel = new JLabel("Dauer Fehleranzeige [ms]");
        // nur ganzzahlige Werte zulassen
        final NumberFormat format = NumberFormat.getIntegerInstance();
        // Tausendertrenner verhindern
        format.setGroupingUsed(false);
        delayTextField = new JFormattedTextField(format);
        // Falscheingabe verhindern
        ((NumberFormatter) delayTextField.getFormatter())
                .setAllowsInvalid(false);
        delayTextField.setColumns(5);
        delayTextField.setText(mainGUI.getMistakeDelay());
        delayTextField.addCaretListener(this);

        // Layout
        final GridBagLayout gridbag = new GridBagLayout();
        final GridBagConstraints c = new GridBagConstraints();
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(5, 5, 5, 5);
        this.setLayout(gridbag);
        this.add(sizeLabel, c, 0, 0, 1, 1, this);
        this.add(slider, c, 1, 0, 1, 1, this);
        this.add(fettLabel, c, 0, 1, 1, 1, this);
        this.add(boldBox, c, 1, 1, 1, 1, this);
        this.add(accessLabel, c, 0, 2, 1, 1, this);
        this.add(contrastBox, c, 1, 2, 1, 1, this);
        this.add(mistakeLabel, c, 0, 3, 1, 1, this);
        this.add(mistakeBox, c, 1, 3, 1, 1, this);
        this.add(successSoundLabel, c, 0, 4, 1, 1, this);
        this.add(successSoundBox, c, 1, 4, 1, 1, this);
        this.add(speechLabel, c, 0, 5, 1, 1, this);
        this.add(speechBox, c, 1, 5, 1, 1, this);
        this.add(delayLabel, c, 0, 6, 1, 1, this);
        this.add(delayTextField, c, 1, 6, 1, 1, this);
    }

    /**
     * Factory method used to construct the gridbag layout
     * 
     * @param c
     *            The Component that has to be layed out in the given container
     * @param constraints
     *            Layout constraints
     * @param x
     *            The constraints gridx value
     * @param y
     *            The constraints gridy value
     * @param w
     *            The constraints gridwidth value
     * @param h
     *            The constraints gridheight value
     * @param p
     *            The container that has to be layed out
     */
    private void add(final Component c, final GridBagConstraints constraints,
            final int x, final int y, final int w, final int h,
            final Container p) {
        constraints.gridx = x;
        constraints.gridy = y;
        constraints.gridwidth = w;
        constraints.gridheight = h;
        p.add(c, constraints);
    }

    @Override
    public void stateChanged(final ChangeEvent arg0) {
        if (arg0.getSource() == slider) {
            mainGUI.setFaktor(slider.getValue());
        }

        if (arg0.getSource() == boldBox) {
            final boolean sel = ((JCheckBox) arg0.getSource()).isSelected();
            mainGUI.setFontWeight(sel);
            mainGUI.repaint();
            this.requestFocus();
        }

        // set contrast and large tooltips
        if (arg0.getSource() == contrastBox) {
            final boolean sel = ((JCheckBox) arg0.getSource()).isSelected();
            mainGUI.setContrast(sel);
            mainGUI.setTooltipFont();
        }

        if (arg0.getSource() == mistakeBox) {
            mainGUI.setMistakeHint(((JCheckBox) arg0.getSource()).isSelected());
        }

        if (arg0.getSource() == successSoundBox) {
            mainGUI.setSuccessSoundPlayable(((JCheckBox) arg0.getSource())
                    .isSelected());
        }

        if (arg0.getSource() == speechBox) {
            mainGUI.setSpeechPlayable(((JCheckBox) arg0.getSource())
                    .isSelected());
        }
    }

    /**
     * Saves the mistake delay time while entered in the text field
     * 
     * @see javax.swing.event.CaretListener#caretUpdate(javax.swing.event.CaretEvent)
     */
    @Override
    public void caretUpdate(final CaretEvent ce) {
        if (ce.getSource() == delayTextField) {
            mainGUI.setMistakeDelay(delayTextField.getText());
        }
    }
}
