/*
 * @StatusBar.java 20.10.2010 Copyright 2010 - 2017 yourwebs.de. All rights
 * reserved
 * 
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Sudoku Sudoku is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * Sudoku is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Spiele; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA 02111-1307, USA.
 */
package de.yourwebs.sudoku.gui;

import java.awt.FlowLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.net.URL;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSeparator;
import javax.swing.JSlider;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import de.yourwebs.sudoku.Config;

/**
 * The status bar of the sudoku game
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
public class StatusBar extends JPanel {
    /**
     * The main frame
     */
    private final SudokuGUI gui;
    /**
     * Label that displays the total number of cells
     */
    private JLabel cellLabel;
    /**
     * Label that displays the numver of mistakes
     */
    private JLabel mistakeLabel;
    /**
     * Slider for adjusting the game's difficulty by increasing or decreasing
     * the initial shown numbers
     */
    private JSlider slider;

    /**
     * Provides an status bar with multiple functionalities
     * 
     * @param gui
     *            <code>SudokuGUI</code> the main window of the application
     * @param initialSliderPosition
     *            <code>int</code> the position of the difficulties slider is
     *            loaded from the config file.
     */
    public StatusBar(final SudokuGUI gui, final int initialSliderPosition) {
        this.gui = gui;
        this.setBorder(new EmptyBorder(0, 10, 0, 10));
        this.createStatusElements(initialSliderPosition);
        this.setMistakeCount(0);
    }

    /**
     * Create the status bar's gui elements
     * 
     * @param initialSliderPosition
     *            <code>int</code> initial position of the slider defining the
     *            game's difficulty.
     */
    private void createStatusElements(final int initialSliderPosition) {
        // Cells
        cellLabel = new JLabel();
        // Mistakes
        mistakeLabel = new JLabel();
        // Slider
        final JPanel sliderPanel = new JPanel(
                new FlowLayout(FlowLayout.LEADING, 0, 0));
        slider = new JSlider(Integer.valueOf(Config.MIN_DIFFICULTY.value),
                Integer.valueOf(Config.MAX_DIFFICULTY.value),
                initialSliderPosition);
        slider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(final ChangeEvent e) {
                if ((slider.getValueIsAdjusting())) {
                    gui.setDifficulty(slider.getValue());
                    gui.initNewSudoku();
                }
            }
        });
        slider.setToolTipText("Schwierigkeit einstellen");
        final JToggleButton noteButt = new JToggleButton();
        final JButton helpButt = new JButton();
        // hint button
        noteButt.setToolTipText("Notizmodus einschalten");
        noteButt.setBorder(new LineBorder(this.getBackground(), 1));
        final URL enabledURL = StatusBar.class.getResource("/img/edit.png");
        final URL disabledURL = StatusBar.class
                .getResource("/img/edit_orange.png");
        if (enabledURL != null && disabledURL != null) {
            final ImageIcon enabledIcon = new ImageIcon(enabledURL);
            final ImageIcon disabledIcon = new ImageIcon(disabledURL);
            noteButt.setIcon(disabledIcon);
            noteButt.setSelectedIcon(enabledIcon);
        }
        // switch between puzzle and hint mode (number view and comment view)
        noteButt.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(final ActionEvent e) {
                if (((JToggleButton) e.getSource()).isSelected()) {
                    noteButt.setToolTipText("Notizmodus ausschalten");
                    slider.setEnabled(false);
                    gui.setHintMode(true);
                    helpButt.setEnabled(false);
                } else {
                    noteButt.setToolTipText("Notizmodus einschalten");
                    slider.setEnabled(true);
                    gui.setHintMode(false);
                    helpButt.setEnabled(true);
                }
                gui.switchView();
            }
        });
        // help button
        helpButt.setToolTipText("Hilfe anzeigen");
        helpButt.setBorder(new LineBorder(this.getBackground(), 1));
        final URL helpURL = StatusBar.class.getResource("/img/help.png");
        if (helpURL != null) {
            final ImageIcon helpIcon = new ImageIcon(helpURL);
            helpButt.setIcon(helpIcon);
        }
        helpButt.addActionListener(new ActionListener() {
            @Override
            public void actionPerformed(final ActionEvent e) {
                gui.help();
            }
        });
        // Layout
        this.setLayout(new FlowLayout(SwingConstants.LEADING));
        this.add(cellLabel);
        this.add(new JSeparator(SwingConstants.VERTICAL));
        this.add(mistakeLabel);
        this.add(new JSeparator(SwingConstants.VERTICAL));
        final JLabel plusLabel = new JLabel("+");
        plusLabel.setToolTipText("leichter");
        sliderPanel.add(plusLabel);
        sliderPanel.add(slider);
        final JLabel minusLabel = new JLabel("-");
        minusLabel.setToolTipText("schwerer");
        sliderPanel.add(minusLabel);
        this.add(sliderPanel);
        this.add(new JSeparator(SwingConstants.VERTICAL));
        this.add(noteButt);
        this.add(helpButt);
    }

    /**
     * Sets the number of empty cells to the status bar's cell label.
     * 
     * @param emptyCells
     *            <code>int</code> the number of empty cells.
     * @param rowElementsCount
     *            <code>int</code> the number of cells in one row or column
     */
    public void setFilledOutCellsLabelText(final int emptyCells,
            final int rowElementsCount) {
        final String s = Integer.valueOf(rowElementsCount * rowElementsCount)
                .toString();
        final String str = "ausgef\u00FCllt: "
                + Integer.valueOf(emptyCells).toString() + "|" + s;
        cellLabel.setText(str);
        cellLabel.setToolTipText(str);
    }

    /**
     * Sets the number of mistakes to the status bar's mistake label
     * 
     * @param mistakes
     *            <code>int</code> number of mistakes
     */
    public void setMistakeCount(final int mistakes) {
        final String str = "Fehler: " + Integer.valueOf(mistakes).toString();
        mistakeLabel.setText(str);
        mistakeLabel.setToolTipText(str);
    }

    /**
     * Sets the value to the status bar's slider
     * 
     * @param value
     *            <code>int</code> the value to be set
     * @param fields
     *            <code>int</code> total number of fields in the game
     */
    void setSliderValue(final int value, final int fields) {
        slider.setValue(fields - value);
    }
}
