/*
 * @SudokuGUI.java 14.10.2010 Copyright 2010 - 2013 yourwebs.de. All rights
 * reserved
 * 
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Sudoku. Sudoku is free software; you can redistribute
 * it and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the License,
 * or (at your option) any later version.
 * 
 * Sudoku is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * Spiele; if not, write to the Free Software Foundation, Inc., 59 Temple Place,
 * Suite 330, Boston, MA 02111-1307, USA.
 */
package de.yourwebs.sudoku.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.KeyEvent;
import java.awt.print.PrinterException;
import java.awt.print.PrinterJob;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.net.URL;
import java.util.Random;
import java.util.Timer;
import java.util.TimerTask;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.DataLine;
import javax.swing.ImageIcon;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import de.yourwebs.sudoku.Config;
import de.yourwebs.sudoku.PropertiesManager;
import de.yourwebs.sudoku.SudokuConstants;
import de.yourwebs.sudoku.SudokuEngine;

/**
 * The class provides the main window of the well known Sudoku number puzzle.
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
public class SudokuGUI extends JFrame {

    /*---------------------- Color Constants -------------------*/
    /**
     * Low contrast default foreground color for dark squares
     */
    private final static Color FG_COLOR = new Color(30, 30, 30);
    /**
     * High contrast default foreground color for light squares
     */
    private final static Color FG_COLOR_LIGHT_ACCESS = new Color(250, 250, 250);
    /**
     * High contrast default foreground color for dark squares
     */
    private final static Color FG_COLOR_DARK_ACCESS = new Color(30, 30, 30);
    /**
     * Low contrast default background color for light squares
     */
    private final static Color BG_COLOR_LIGHT = Color.WHITE;
    /**
     * High contrast default background color for light squares
     */
    private final static Color BG_COLOR_LIGHT_ACCESS = Color.WHITE;
    /**
     * Low contrast default background color for dark squares
     */
    private final static Color BG_COLOR_DARK = new Color(200, 200, 200);
    /**
     * High contrast default background color for dark squares
     */
    private final static Color BG_COLOR_DARK_ACCESS = Color.BLACK;
    /**
     * Default border color
     */
    private final static Color BORDER_COLOR = new Color(110, 110, 110);
    /**
     * High contrast default border color for light squares
     */
    private final static Color BORDER_COLOR_LIGHT_ACCESS = new Color(250, 250,
            250);
    /**
     * High contrast default border color for dark squares
     */
    private final static Color BORDER_COLOR_DARK_ACCESS = new Color(30, 30, 30);
    /**
     * Low contrast mistake background color
     */
    private final static Color MISTAKE_BG_COLOR = new Color(255, 230, 210);
    /**
     * High contrast mistake background color
     */
    private final static Color MISTAKE_BG_COLOR_ACCESS = new Color(255, 190,
            180);
    /**
     * Low contrast mistake foreground color
     */
    private final static Color MISTAKE_FG_COLOR = new Color(30, 30, 30);
    /**
     * High contrast mistake foreground color
     */
    private final static Color MISTAKE_FG_COLOR_ACCESS = Color.BLACK;
    /**
     * Highlight background color as shown by fields with identical numbers
     */
    private final static Color HIGHLIGHT_BG_COLOR = new Color(100, 205, 100);

    /**
     * Background color of the light squares depending on contrast settings.<br>
     * 
     * @see #switchContrastColors(boolean)
     * @see #BG_COLOR_LIGHT_ACCESS
     * @see #BG_COLOR_LIGHT
     */
    private Color lightBgColor = BG_COLOR_LIGHT;

    /**
     * Background color of the dark squares depending on contrast settings.<br>
     * 
     * @see #switchContrastColors(boolean)
     * @see #BG_COLOR_DARK_ACCESS
     * @see #BG_COLOR_DARK
     */
    private Color darkBgColor = BG_COLOR_DARK;

    /**
     * Foreground color of the light squares. In normal mode there is no
     * difference between light and dark squares.<br>
     * 
     * @see #switchContrastColors(boolean)
     * @see #FG_COLOR_DARK_ACCESS
     * @see #FG_COLOR
     */
    private Color lightFgColor = FG_COLOR;

    /**
     * Foreground color of the dark squares. In normal mode there is no
     * difference between light and dark squares.<br>
     * 
     * @see #switchContrastColors(boolean)
     * @see #FG_COLOR_LIGHT_ACCESS
     * @see #FG_COLOR
     */
    private Color darkFgColor = FG_COLOR;

    /**
     * Border color of the dark squares in accessibility mode. In normal mode
     * the default color ist displayed.
     * 
     * @see #switchContrastColors(boolean)
     * @see #BORDER_COLOR_LIGHT_ACCESS
     * @see #BORDER_COLOR
     */
    private Color borderColorLight = BORDER_COLOR;

    /**
     * Border color of the light squares in accessibility mode. In normal mode
     * the default color ist displayed.
     * 
     * @see #switchContrastColors(boolean)
     * @see #BORDER_COLOR_DARK_ACCESS
     * @see #BORDER_COLOR
     */
    private Color borderColorDark = BORDER_COLOR;

    /*---------------------------- Default settings ------------------------*/

    /**
     * Number of cells in a square's row or column. Defaults to 3.
     */
    static final int SQUARE_ROW_ELEMENTS = 3;

    /**
     * Number of rows or columns in the entire game.
     */
    private static final int ELEMENTS_IN_ROW = (int) Math
            .pow(SQUARE_ROW_ELEMENTS, 2);

    /**
     * Number of elements in the entire game.
     */
    private static final int ELEMENTS_IN_GAME = (int) Math
            .pow(ELEMENTS_IN_ROW, 2);

    /**
     * Number of empty cells at startup. Lower numbers make the game more easy.
     */
    private int difficulty = SudokuConstants.DIFFICULTY;

    /**
     * Milliseconds a mistake is highlighted before resetting the display to the
     * default appearance after entering a wrong number.
     */
    private int mistakeDelay = SudokuConstants.MISTAKE_DELAY;

    /**
     * Default font weight of the entered numbers
     */
    private int fontWeight = SudokuConstants.FONT_WEIGHT;

    /**
     * Scales the font and border size. A higher number means a smaller font.
     */
    private final int fontScale = SudokuConstants.FONT_SCALE;

    /**
     * Show accessibility mode. Will be changed by the accessibility settings.
     */
    private boolean contrast = SudokuConstants.ACCESSIBILITY;

    /**
     * Give a hint if a false number is entered.
     */
    private boolean mistakeHint = SudokuConstants.MISTAKE_HINT;

    /**
     * Play a sound when finishing the game successfully
     */
    private boolean isSuccessSoundPlayable = SudokuConstants.PLAY_SUCCESS_SOUND;

    /**
     * Play a voice output if <code>true</code>.
     */
    private boolean isSpeechPlayable = SudokuConstants.PLAY_VOICE_OUTPUT;

    /**
     * Count the number of mistakes to display it on the status bar.
     */
    private int mistakes = 0;

    /**
     * Used to scale font and gui by the application's options dialog's slider.
     * Initialized by the default size of the frame set by the constructor.
     */
    private int faktor = 200;

    /**
     * The game's mode, <code>true</code> if the hint mode is enabled otherwise
     * <code>false</code>. @see #help()
     */
    private boolean hintMode = false;

    /**
     * Array of all element cells ordered from the upper left to the lower right
     * 
     * @see de.yourwebs.sudoku.gui.Element
     */
    private Element[][] elements;

    /**
     * The game's status bar
     * 
     * @see de.yourwebs.sudoku.gui.StatusBar
     */
    private StatusBar statusBar;

    /**
     * The program icon
     */
    private final ImageIcon icon;

    /**
     * The font used to display tooltips
     */
    private final Font defaultTooltipFont;

    /**
     * The business logic
     * 
     * @see de.yourwebs.sudoku.SudokuEngine
     */
    private final SudokuEngine engine;

    /**
     * The configuration settings are stored in properties.
     * 
     * @see de.yourwebs.sudoku.PropertiesManager
     */
    private PropertiesManager pm;

    /**
     * A printable <code>JPanel</code>
     * 
     * @see de.yourwebs.sudoku.gui.PrintablePanel
     */
    private PrintablePanel mainPanel;

    /**
     * Creates the game's main frame
     */
    public SudokuGUI() {
        final URL url = this.getClass().getClassLoader()
                .getResource(Config.ICON_PATH.value);
        icon = new ImageIcon(url);
        this.setIconImage(icon.getImage());
        defaultTooltipFont = UIManager.getFont("ToolTip.font");
        this.loadProperties();

        engine = new SudokuEngine();
        engine.initEngine(SQUARE_ROW_ELEMENTS);
        engine.createSudoku(0, 0);
        engine.preallocate(difficulty);

        this.addComponentListener(new SudokuComponentAdapter(this));

        this.initGUI();
        this.makeConfigFile();

        this.addComponentListener(new ComponentAdapter() {
            @Override
            public void componentResized(final ComponentEvent e) {
                faktor = ((JFrame) e.getSource()).getSize().width
                        / SQUARE_ROW_ELEMENTS;
                SudokuGUI.this.setFaktor(faktor);
                SudokuGUI.this.reloadElementFont();
                SudokuGUI.this.repaint();
            }
        });
        this.setVisible(true);
    } // Konstruktor

    /**
     * Creates and initializes the gui
     */
    private void initGUI() {
        this.setTooltipFont();
        this.setLayout(new BorderLayout());
        elements = new Element[ELEMENTS_IN_ROW][ELEMENTS_IN_ROW];
        this.initMainPanel();
        this.initStatusBar();
        this.initMenubar();
        this.switchContrastColors(contrast);
        this.setElementsDefaultColors();
        this.setComponentSizeAndPosition(this);
        this.setTitle(Config.APP_NAME.value);
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.resizeGUI(faktor);
    }

    /**
     * Creates the main panel of the game. An array of elements is created and
     * filled by numbers. These are fetched from an int-array, where -1 means an
     * empty field. Finally the elements are set to the panel.
     * 
     * @see de.yourwebs.sudoku.SudokuEngine#getStartArray()
     */
    private void initMainPanel() {
        if (engine == null) {
            return;
        }
        mainPanel = new PrintablePanel(
                new GridLayout(ELEMENTS_IN_ROW, ELEMENTS_IN_ROW, 0, 0));
        String s = "";
        final byte intArr[][] = engine.getStartArray();
        if (intArr == null) {
            return;
        }
        for (int i = 0; i < ELEMENTS_IN_ROW; i++) {
            for (int j = 0; j < ELEMENTS_IN_ROW; j++) {
                elements[i][j] = new Element(this, faktor / fontScale,
                        fontWeight);
                final String fw = pm.getProperty("fontWeight");
                if (fw != null && !fw.equals("")) {
                    elements[i][j]
                            .setFontWeight(Integer.valueOf(fw).intValue());
                }
                // defines the positions of the elements in squares, rows and
                // columns
                final int quad = (i / SQUARE_ROW_ELEMENTS) * SQUARE_ROW_ELEMENTS
                        + j / SQUARE_ROW_ELEMENTS;
                elements[i][j].setQuad(quad);
                elements[i][j].setRow(i);
                elements[i][j].setCol(j);
                // set the number to the elements field
                s = (Integer.valueOf(intArr[i][j]).toString());
                s = s.equals("-1") ? "" : s;
                elements[i][j].setFieldText(s);
                mainPanel.add(elements[i][j]);
            }
        }
        this.add(mainPanel, BorderLayout.CENTER);
    }

    /**
     * Create a menuBar
     */
    private void initMenubar() {
        final JMenuBar menuBar = new SudokuMenu(this);
        this.setJMenuBar(menuBar);
    }

    /**
     * Create a status bar
     */
    private void initStatusBar() {
        statusBar = new StatusBar(this, difficulty);
        // show the number of empty cells in the statusBar bar.
        statusBar.setFilledOutCellsLabelText(this
                .getFilledCellsCount(), ELEMENTS_IN_ROW);
        this.add(statusBar, BorderLayout.SOUTH);
    }

    /**
     * Reloads the font of all elements if the component is resized or the
     * font-weight has changed.
     */
    void reloadElementFont() {
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                elements[i][j].reloadFont();
            }
        }
    }

    /**
     * Repaints the gui depending on <code>faktor</code>. Called when the frame
     * is resized.
     * 
     * @param faktor
     *            <code>int</code>
     */
    void resizeGUI(final int faktor) {
        this.setFaktor(faktor);
        this.reloadElementFont();
        this.repaint();
    }

    /**
     * Sets the component's size depending on {@link #faktor} and centers it on
     * the screen.
     * 
     * @param frame,
     *            the <code>JFrame</code> to be handled
     */
    private void setComponentSizeAndPosition(final JFrame frame) {
        frame.setSize(SQUARE_ROW_ELEMENTS * faktor, SQUARE_ROW_ELEMENTS
                * faktor);
        frame.setLocationRelativeTo(null);
    }

    /**
     * Loads the game's properties from the configuration file.
     */
    private void loadProperties() {
        pm = new PropertiesManager();
        pm.loadProperties();
        if (pm.getProperty("sizeSlider") != null
                && !pm.getProperty("sizeSlider").equals("")) {
            faktor = Integer.valueOf(pm.getProperty("sizeSlider"));
        }
        if (pm.getProperty("difficulty") != null
                && !pm.getProperty("difficulty").equals("")) {
            difficulty = Integer.valueOf(pm.getProperty("difficulty"));
        }
        if (pm.getProperty("fontWeight") != null
                && !pm.getProperty("fontWeight").equals("")) {
            fontWeight = Integer.valueOf(pm.getProperty("fontWeight"))
                    .intValue();
        }
        if (pm.getProperty("contrast") != null
                && !pm.getProperty("contrast").equals("")) {
            contrast = Boolean.valueOf(pm.getProperty("contrast"))
                    .booleanValue();
        }
        if (pm.getProperty("mistakeHint") != null
                && !pm.getProperty("mistakeHint").equals("")) {
            mistakeHint = Boolean.valueOf(pm.getProperty("mistakeHint"))
                    .booleanValue();
        }
        if (pm.getProperty("isSuccessSoundPlayable") != null
                && !pm.getProperty("isSuccessSoundPlayable").equals("")) {
            isSuccessSoundPlayable = Boolean
                    .valueOf(pm.getProperty("isSuccessSoundPlayable"))
                    .booleanValue();
        }
        if (pm.getProperty("isSpeechPlayable") != null
                && !pm.getProperty("isSpeechPlayable").equals("")) {
            isSpeechPlayable = Boolean
                    .valueOf(pm.getProperty("isSpeechPlayable")).booleanValue();
        }
        if (pm.getProperty("mistakeDelay") != null
                && !pm.getProperty("mistakeDelay").equals("")) {
            mistakeDelay = Integer.valueOf(pm.getProperty("mistakeDelay"))
                    .intValue();
        }
    }

    /**
     * Counts the number of cells filled with digits.
     * 
     * @return <code>int</code> the number of not empty cells
     */
    int getFilledCellsCount() {
        int counter = 0;
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                if (elements[i][j].getFieldText().equals("")) {
                    counter++;
                }
            }
        }
        return ELEMENTS_IN_GAME - counter;
    }

    /**
     * Manages the activity of the help button. If pressed a single empty field
     * is filled by the digit according to the result array's solution.
     */
    void help() {
        final Element[][] elements = this.getElements();
        // get an empty random field
        final Random r = new Random();
        final int i = r.nextInt(ELEMENTS_IN_ROW);
        final int j = r.nextInt(ELEMENTS_IN_ROW);
        final byte[][] arr = engine.getResultArray();
        // get the random value in the result array
        final byte value = arr[i][j];
        // recursion if the element is not empty
        if (!elements[i][j].getFieldText().equals("")
                && this.getFilledCellsCount() < ELEMENTS_IN_GAME) {
            this.help();
            return;
        }
        // set value to the text field
        this.checkForMistake(elements[i][j], value);
        final String s = Integer.valueOf(value).toString();
        if (!hintMode) {
            elements[i][j].setFieldText(s);
            this.showSuccess(s);
        }
        statusBar.setFilledOutCellsLabelText(this
                .getFilledCellsCount(), ELEMENTS_IN_ROW);
        statusBar.setSliderValue(this
                .getFilledCellsCount(), (int) Math.pow(ELEMENTS_IN_ROW, 2));

    }

    /**
     * Reloads a previously saved Sudoku from {@link Config#CONFIG_DIR} which is
     * saved in a hidden file {@link de.yourwebs.sudoku.Config#SAVE_FILE}.<br>
     * The file contains two digits in one line. The first number shows the
     * stored result if any, the second one is the expected digit pursuant to
     * the {@link de.yourwebs.sudoku.SudokuEngine#resultArray}.
     * 
     * @see #save()
     */
    void load() {
        if (this.makeConfigFile()) {
            String s, startStr = null, resStr = null;
            BufferedReader br = null;
            byte[][] startArray = null;
            byte[][] resultArray = null;
            final File f = new File(Config.SAVE_FILE.value);
            if (f.length() > 0) {
                try {
                    startArray = new byte[elements.length][elements.length];
                    resultArray = new byte[elements.length][elements.length];
                    br = new BufferedReader(new FileReader(f));
                    for (int i = 0; i < elements.length; i++) {
                        for (int j = 0; j < elements[i].length; j++) {
                            startStr = " ";
                            resStr = " ";
                            elements[i][j].setFieldText("");
                            startArray[i][j] = -1;
                            resultArray[i][j] = -1;
                            s = br.readLine();
                            if (s.length() > 1) {
                                startStr = s.substring(0, 1);
                                resStr = s.substring(1, 2);
                            }
                            if (startStr != null && !startStr.equals("")
                                    && !startStr.equals(" ")) {
                                elements[i][j].setFieldText(startStr);
                                startArray[i][j] = Byte.parseByte(startStr);
                            }
                            this.highlightNumber("-1");
                            // Result-Array beladen
                            if (!resStr.equals(" ")) {
                                resultArray[i][j] = Byte.parseByte(resStr);
                            }
                        }
                    }
                    statusBar.setMistakeCount(0);

                } catch (final FileNotFoundException e1) {
                    e1.printStackTrace();
                } catch (final IOException e) {
                    e.printStackTrace();
                } finally {
                    if (br != null) {
                        try {
                            br.close();
                        } catch (final IOException e) {
                        }
                    }
                }
                if (startArray != null && resultArray != null) {
                    engine.setStartArray(startArray);
                    engine.setResultArray(resultArray);
                    for (int i = 0; i < elements.length; i++) {
                        for (int j = 0; j < elements[i].length; j++) {
                            this.checkForMistake(elements[i][j], startArray[i][j]);
                        }
                    }
                    statusBar.setFilledOutCellsLabelText(this
                            .getFilledCellsCount(), ELEMENTS_IN_ROW);
                    statusBar.setSliderValue(this
                            .getFilledCellsCount(), (int) Math
                                    .pow(ELEMENTS_IN_ROW, 2));
                }
            } else {
                JOptionPane
                        .showMessageDialog(this, "Es ist noch kein Sudoku gespeichert.\n"
                                + "Laden Sie erst nach dem Abspeichern eines Spiels.", "Laden nicht m\u00F6glich!", JOptionPane.OK_OPTION, icon);
            }
        } else {
            System.err.println("Konfigurationsdatei kann nicht angelegt werden!"
                    + "\n" + "Laden der Einstellungen nicht m\u00F6glich!");
        }
    }

    /**
     * Saves the current state and the result values of the game in
     * {@link de.yourwebs.sudoku.Config#SAVE_FILE} located in the sudoku's
     * configuration directory. Each element is stored in a separate line in a
     * two digit number. The second digit represent the result array's value,
     * the first one represents the current entry in the specified field.
     * 
     * @see #load()
     */
    void save() {
        // create a reference to the result array
        final byte[][] resultArray = engine.getResultArray();
        // PrintWriter pWriter = null;
        String s, fieldTxt;
        int resNum;
        File file = new File(Config.SAVE_FILE.value);
        String line = null;
        // temp. Array that stores all written values for comparing with readArr
        String[] saveArr = new String[ELEMENTS_IN_GAME];
        // temp. Array that stores all read values for comparing with saveArr
        String[] readArr = new String[ELEMENTS_IN_GAME];
        try (PrintWriter pWriter = new PrintWriter(
                new FileWriter(Config.SAVE_FILE.value))) {
            
            for (int i = 0; i < elements.length; i++) {
                for (int j = 0; j < elements[i].length; j++) {
                    // get the field's result array value
                    resNum = resultArray[i][j];
                    // get the field's value
                    fieldTxt = elements[i][j].getFieldText();
                    // print a blank if no value ist set
                    if (fieldTxt.equals("") || fieldTxt == null) {
                        fieldTxt = " ";
                    }
                    // concatenate the field value as first digit and the
                    // result value as second digit
                    s = fieldTxt + Integer.valueOf(resNum).toString();
                    pWriter.println(s);
                    // create an array for comparing with actually saved values
                    saveArr[((i* elements.length) + (j+1))-1] = s;
                }
            }
        } catch (final IOException ioe) {
            ioe.printStackTrace();
        }
        // Read store file line by line
        try(BufferedReader br = new BufferedReader(new FileReader(file))) {
            int i = 0;
            while((line = br.readLine()) != null) {
                readArr[i] = line;
                i++;
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
        // Compare both arrays and print a message
        for(int i=0; i<ELEMENTS_IN_GAME; i++) {
            if(!saveArr[i].equals(readArr[i])) {
                JOptionPane.showMessageDialog(this, "Speichern fehlgeschlagen!", "Fehler", JOptionPane.OK_OPTION, icon);
                return;
            }
        }
        JOptionPane.showMessageDialog(this, "Speichern erfolgreich!", "Erfolg", JOptionPane.OK_OPTION, icon);
    }

    /**
     * Prints the current state of the game's {@link #mainPanel} as image.
     */
    public void print() {
        final PrinterJob printJob = PrinterJob.getPrinterJob();
        printJob.setPrintable(mainPanel);
        if (printJob.printDialog()) {
            try {
                printJob.print();
            } catch (final PrinterException pe) {
                System.out.println("Error printing: " + pe);
            }
        }
    }

    /**
     * Creates a file to save the settings of the game
     * 
     * @return <code>true</code> if the file was successfully created or exists
     *         already.
     */
    private boolean makeConfigFile() {
        try {
            final File dir = new File(Config.CONFIG_DIR.value);
            if (!dir.isDirectory() && !dir.mkdir()) {
                return false;
            }
            final File cf = new File(Config.SAVE_FILE.value);

            if (cf.isFile() && cf.canRead() && cf.canWrite()) {
                return true;
            }
            if (cf.createNewFile()) {
                return true;
            }
        } catch (final IOException e) {
            e.printStackTrace();
        }
        return false;
    }

    /********************** Accessors ********************/

    /**
     * 
     * @return <code>Element[][]</code> all elements organized as
     *         Element[row][column]
     */
    public Element[][] getElements() {
        return elements;
    }

    /**
     * 
     * @return <code>int</code> the factor that is used for resizing the game
     */
    public int getFaktor() {
        return faktor;
    }

    /**
     * Sets the factor for resizing the the game's gui including the font scale
     * of the elements. It's saved in the game's properties.
     * 
     * @param faktor
     *            <code>int</code> the numeric factor used for resizing
     * @see #setComponentSizeAndPosition(JFrame)
     */
    void setFaktor(final int faktor) {
        if (this.faktor != faktor) {
            this.faktor = faktor;
        }
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                elements[i][j].setFaktor(faktor / fontScale);
            }
        }
        pm.setProperty("sizeSlider", Integer.valueOf(faktor).toString());
        pm.saveProperties();
        this.setComponentSizeAndPosition(this);
    }

    /**
     * 
     * @return <code>int</code> the font weight
     * @see de.yourwebs.sudoku.SudokuConstants#FONT_WEIGHT
     */
    int getFontWeight() {
        return fontWeight;
    }

    /**
     * Controlls the game's font weight
     * 
     * @param weight
     *            - if <code>true</code> the font is shown bold, otherwise plain
     * 
     * @see #fontWeight
     */
    public void setFontWeight(final boolean weight) {
        fontWeight = weight ? Font.BOLD : Font.PLAIN;
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                elements[i][j].setFontWeight(fontWeight);
            }
        }
        this.reloadElementFont();
        pm.setProperty("fontWeight", Integer.valueOf(fontWeight).toString());
        pm.saveProperties();
    }

    /**
     * Provides the current contrast settings. If <code>true</code> a high
     * contrast mode is shown.
     * 
     * @return <code>boolean</code> the current contrast settings
     */
    boolean getContrast() {
        return contrast;
    }

    /**
     * Set the accessible mode with high contrast
     * 
     * @param selected
     *            <code>boolean</code>, checkbox statusBar in options dialog
     */
    public void setContrast(final boolean selected) {
        this.switchContrastColors(selected);
        pm.setProperty("contrast", Boolean.valueOf(selected).toString());
        pm.saveProperties();
        this.setElementsDefaultColors();
    }

    /**
     * Provides the current mistake hint settings. If <code>true</code> the row,
     * column and square where the mistake occures, are highlighted for a time,
     * which is defined by
     * {@link de.yourwebs.sudoku.SudokuConstants#MISTAKE_DELAY}
     * 
     * @return <code>boolean, true</code> if the hints are shown
     */
    boolean getMistakeHint() {
        return mistakeHint;
    }

    /**
     * Display a hint if a false value is inserted
     * 
     * @param selected
     *            <code>boolean</code>, checkbox statusBar in options dialog
     */
    void setMistakeHint(final boolean selected) {
        mistakeHint = selected;
        pm.setProperty("mistakeHint", Boolean.valueOf(selected).toString());
        pm.saveProperties();
    }

    /**
     * 
     * @return <code>boolean</code> if <code>true</code> a success sound will be
     *         played when the game is finished successfully
     */
    boolean getSuccessSoundPlayable() {
        return isSuccessSoundPlayable;
    }

    /**
     * If <code>true</code> a sound is played, when the game is finished
     * successfully. The value is set to {@link #isSuccessSoundPlayable} and
     * stored within the game's properties.
     * 
     * @param aku
     *            <code>boolean</code>
     */
    void setSuccessSoundPlayable(final boolean aku) {
        isSuccessSoundPlayable = aku;
        pm.setProperty("isSuccessSoundPlayable", Boolean.valueOf(aku)
                .toString());
        pm.saveProperties();
    }

    /**
     * 
     * @return <code>boolean</code> if <code>true</code> the entered digit is
     *         spoken
     */
    boolean getSpeechPlayable() {
        return isSpeechPlayable;
    }

    /**
     * If <code>true</code> while setting a number a sound is played speaking
     * the entered digit. The value is set to {@link #isSuccessSoundPlayable}
     * and stored within the game's properties.
     * 
     * @param speech
     *            <code>boolean, true</code> if a speech sound should be played
     */
    void setSpeechPlayable(final boolean speech) {
        isSpeechPlayable = speech;
        pm.setProperty("isSpeechPlayable", Boolean.valueOf(speech).toString());
        pm.saveProperties();
    }

    /**
     * 
     * @return <code>String</code> the number of milliseconds a mistake hint is
     *         shown.
     */
    String getMistakeDelay() {
        return Integer.valueOf(mistakeDelay).toString();
    }

    /**
     * Saves the time settings mistake hints are shown.
     * 
     * @param delay,
     *            a <code>String</code>, denoting the milliseconds the mistake
     *            hints are shown.
     */
    void setMistakeDelay(final String delay) {
        if (!delay.equals("") && delay.matches("[0-9]+")) {
            mistakeDelay = Integer.valueOf(delay).intValue();
            pm.setProperty("mistakeDelay", delay);
            pm.saveProperties();
        }
    }

    /**
     * 
     * @return the hint mode, that is, if hints are shown or not
     */
    boolean getHintMode() {
        return hintMode;
    }

    /**
     * Sets the game's mode. If <code>true</code> the game is played in hint
     * mode. That means that one field can display multiple digits i.e. for
     * memory reasons. If <code>false</code> the normal puzzle mode is
     * displayed.
     * 
     * @param mode
     *            <code>boolean</code> the game's mode
     */
    void setHintMode(final boolean mode) {
        hintMode = mode;
    }

    /**
     * 
     * @return <code>int</code> the difficulty of the game
     * 
     * @see #difficulty
     */
    public int getDifficulty() {
        return difficulty;
    }

    /**
     * Sets the {@link #difficulty} of the game
     * 
     * @param difficulty
     *            <code>int</code> the difficulty
     */
    void setDifficulty(final int difficulty) {
        this.difficulty = difficulty;
        pm.setProperty("difficulty", Integer.valueOf(difficulty).toString());
        pm.saveProperties();
    }

    /**
     * 
     * @return <code>int</code> the number of elements in one row/column
     * 
     * @see #ELEMENTS_IN_ROW
     */
    public int getElementsInRow() {
        return ELEMENTS_IN_ROW;
    }

    /**
     * 
     * @return <code>SudokuEngine</code>, an instance of the game's business
     *         logic
     */
    public SudokuEngine getEngine() {
        return engine;
    }

    /**
     * Sets the tooltip font depending on {@link #contrast}.<br>
     * If <code>true</code> the game's tooltips are shown much larger according
     * to {@link SudokuConstants#TOOLTIP_FONT} otherwise the default font is
     * loaded.
     * 
     */
    public void setTooltipFont() {
        if (contrast) {
            UIManager.put("ToolTip.font", SudokuConstants.TOOLTIP_FONT);
        } else {
            UIManager.put("ToolTip.font", defaultTooltipFont);
        }
        this.repaint();
    }

    /**
     * Play <code>soundFile</code> when a digit is enterd or the game is
     * finished successfully
     * 
     * @param soundFile
     *            <code>String</code> the name of the sound file
     */
    void playSound(final String soundFile) {
        try {
            final URL url = this.getClass().getClassLoader()
                    .getResource(soundFile);
            if (url == null) {
                return;
            }
            final AudioInputStream audioInputStream = AudioSystem
                    .getAudioInputStream(url);
            final AudioFormat af = audioInputStream.getFormat();
            Clip clip = null;
            final DataLine.Info info = new DataLine.Info(Clip.class,
                    audioInputStream.getFormat(),
                    ((int) audioInputStream.getFrameLength()
                            * af.getFrameSize()));

            clip = (Clip) AudioSystem.getLine(info);
            clip.open(audioInputStream);
            clip.start();
        } catch (final Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Create a new Sudoku game
     */
    void initNewSudoku() {
        if (engine == null) {
            return;
        }
        this.clearHintLabels();
        this.setElementsDefaultColors();
        engine.initFields();
        engine.createSudoku(0, 0);
        engine.preallocate(this.getDifficulty());
        this.updateSudoku(engine.getStartArray());
        statusBar.setFilledOutCellsLabelText(this
                .getFilledCellsCount(), ELEMENTS_IN_ROW);
        mistakes = 0;
        statusBar.setMistakeCount(mistakes);
    }

    /**
     * Generates a new set of numbers and displays them on the element's
     * textfields.
     * 
     * @param arr
     *            <code>byte[][]</code> the number array used to initialize the
     *            game.
     */
    void updateSudoku(final byte[][] arr) {
        if (engine == null) {
            return;
        }
        if (arr.equals(engine.getResultArray())) {
            engine.setStartArray(engine.getResultArray());
            this.setElementsDefaultColors();
        }
        String s;
        for (int i = 0; i < ELEMENTS_IN_ROW; i++) {
            for (int j = 0; j < ELEMENTS_IN_ROW; j++) {
                s = (Integer.valueOf(arr[i][j]).toString());
                s = s.equals("-1") ? "" : s;
                elements[i][j].setFieldText(s);
            }
        }
    }

    /**
     * Opens a dialog that shows the completed sudoku
     */
    void showSolution() {
        final JDialog f = new JDialog(this, "L\u00f6sung", false);
        f.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);

        PrintablePanel mainPanel = new PrintablePanel(
                new GridLayout(ELEMENTS_IN_ROW, ELEMENTS_IN_ROW, 0, 0));
        final byte[][] b = engine.getResultArray();
        final Element[][] el = new Element[ELEMENTS_IN_ROW][ELEMENTS_IN_ROW];
        for (int i = 0; i < ELEMENTS_IN_ROW; ++i) {
            for (int j = 0; j < ELEMENTS_IN_ROW; ++j) {
                el[i][j] = new Element(this, faktor / fontScale, fontWeight);
                el[i][j].setFieldText(Integer.valueOf(b[i][j]).toString());
                el[i][j].reloadFont();
                mainPanel.add(el[i][j]);
            }
        }
        f.add(mainPanel);
        f.setSize(SQUARE_ROW_ELEMENTS * faktor, SQUARE_ROW_ELEMENTS * faktor);
        f.setVisible(true);
    }

    /*************************** Success handling *************************/

    /**
     * Sums up all entry values row by row and compares each row's result with
     * the sum of 1 to {@link #ELEMENTS_IN_ROW}.
     * 
     * @return <code>true</code> if all the rows are filled out properly so that
     *         each row contains one example of all required numbers.
     */
    private boolean verifySuccess() {
        boolean handle = false;
        String s;
        int n, sum = 0;
        final int entryValues = engine.getSumOfFieldValues();
        for (int i = 0; i < elements.length; i++) {
            handle = false;
            sum = 0;
            for (int j = 0; j < elements[i].length; j++) {
                s = elements[i][j].getFieldText();
                if (s.equals("")) {
                    continue;
                }
                n = Integer.valueOf(s).intValue();
                sum += n;
                if (sum == entryValues) {
                    handle = true;
                } else {
                    handle = false;
                }
            }
            if (handle == false) {
                return false;
            }
        }
        return handle;
    }

    /**
     * If enabled a sound is played when the correct number is written or if the
     * game ist finished correctly. In case of a finished game, an additional
     * dialog ist displayed.
     * 
     * @param s
     *            <code>String</code>, the name of the number's sound file
     *            whithout extension.
     * 
     * @see #playSound(String)
     * @see #isSpeechPlayable
     * @see #isSuccessSoundPlayable
     */
    private void showSuccess(final String s) {
        if (isSpeechPlayable && s != null) {
            this.playSound(Config.SOUND_PATH.value + s + ".wav");
        }
        if (this.verifySuccess()) {
            if (isSuccessSoundPlayable) {
                this.playSound(Config.SUCCESS_SOUND.value);
            }
            new SuccessDialog(this);
        } else {

        }
    }

    /********************** Mistake handling *******************/

    /**
     * Activates a timer, that schedules the reset of the gui after showing a
     * mistake. The mistake is shown by changing the background-color of the
     * affected row/col/square.<br>
     * 
     * @see de.yourwebs.sudoku.gui.SudokuGUI#highlightMistake(int, int, int,
     *      int, int)
     * 
     * @param el
     *            <code>Element</code>, the Element-Object causing the mistake.
     * 
     */
    private void handleMistake(final Element el) {
        if (engine == null) {
            return;
        }
        // Remove the wrong entry from the start array
        engine.getStartArray()[el.getRow()][el.getCol()] = 0;
        final Timer t = new Timer();
        final MistakeTimer mt = new MistakeTimer(this, el);
        t.schedule(mt, mistakeDelay);
        statusBar.setMistakeCount(++mistakes);
    }

    /**
     * Responsible for the highlighting of the participating row/col/square if a
     * mistake occurs. A mistake is detected by
     * {@link #checkForMistake(Element, byte)}. According to the location of its
     * occurance, whether the double entry is in a row, a column, square or
     * multiple of these, <code>mistake</code> is given and the associated
     * fields are highlighted by <code>{@link MISTAKE_FG_COLOR}</code> and
     * {@link MISTAKE_BG_COLOR}.
     * 
     * @param row
     *            <code>int</code>, row number starting with '0'
     * @param col
     *            <code>int</code>, column number starting with '0'
     * @param quad
     *            <code>int</code>, square number starting with '0'
     * @param mistake
     *            <code>int</code>, number of the kind of local match
     * 
     *            <dl>
     *            <dt>0</dt>
     *            <dd>horizontal match</dd>
     *            <dt>1</dt>
     *            <dd>vertical match</dd>
     *            <dt>2</dt>
     *            <dd>horizontal and vertical match</dd>
     *            <dt>3</dt>
     *            <dd>match in a square</dd>
     *            <dt>4</dt>
     *            <dd>horizontal match and match in a square</dd>
     *            <dt>5</dt>
     *            <dd>vertical match and match in a square</dd>
     *            <dt>6</dt>
     *            <dd>horizontal and vertical match and match in a square</dd>
     *            <dt>-1</dt>
     *            <dd>default</dd>
     *            </dl>
     * @param value
     *            <code>int</code>, Wert der falschen Zahl
     * 
     * @see de.yourwebs.sudoku.SudokuEngine#check(int, int, int)
     */
    private void highlightMistake(final int row, final int col, final int quad,
            final int mistake, final int value) {
        // Vordergrundfarbe des falschen Zeichens einstellen
        Color fgColor = MISTAKE_FG_COLOR;
        Color bgColor = MISTAKE_BG_COLOR;
        if (contrast) {
            fgColor = MISTAKE_FG_COLOR_ACCESS;
            bgColor = MISTAKE_BG_COLOR_ACCESS;
        }
        // Iterate over all elements and mark multiple occurences
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                switch (mistake) {
                case 0:
                    if (elements[i][j].getRow() == row) {
                        elements[i][j].setBackground(bgColor, bgColor);
                        if (elements[i][j].getFieldText().trim()
                                .equals(Integer.valueOf(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 1:
                    if (elements[i][j].getCol() == col) {
                        elements[i][j].setBackground(bgColor, bgColor);
                        if (elements[i][j].getFieldText().trim()
                                .equals(Integer.valueOf(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 2:
                    if (elements[i][j].getRow() == row
                            || elements[i][j].getCol() == col) {
                        elements[i][j].setBackground(bgColor, bgColor);
                        if (elements[i][j].getFieldText().trim()
                                .equals(Integer.valueOf(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 3:
                    if (elements[i][j].getQuad() == quad) {
                        elements[i][j].setBackground(bgColor, bgColor);
                        if (elements[i][j].getFieldText().trim()
                                .equals(Integer.valueOf(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 4:
                    if (elements[i][j].getRow() == row
                            || elements[i][j].getQuad() == quad) {
                        elements[i][j].setBackground(bgColor, bgColor);
                        if (elements[i][j].getFieldText().trim()
                                .equals(Integer.valueOf(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 5:
                    if (elements[i][j].getCol() == col
                            || elements[i][j].getQuad() == quad) {
                        elements[i][j].setBackground(bgColor, bgColor);
                        if (elements[i][j].getFieldText().trim()
                                .equals(Integer.valueOf(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                case 6:
                    if (elements[i][j].getCol() == col
                            || elements[i][j].getRow() == row
                            || elements[i][j].getQuad() == quad) {
                        elements[i][j].setBackground(bgColor, bgColor);
                        if (elements[i][j].getFieldText().trim()
                                .equals(Integer.valueOf(value).toString())) {
                            elements[i][j].setForeground(fgColor);
                        }
                    }
                    break;
                default:
                    return;
                }

                elements[i][j]
                        .setSelectionColor(elements[i][j].getBackground());
            }
        }
    }

    /**
     * Checks in <code>b</code> exists in row, column or square to which
     * <code>el</code> belongs. If mistake hints are to be shown, the row,
     * column or square containing the double number will be highlighted.
     * 
     * @param el
     *            The <code>Element</code> belonging to the row, column or
     *            square with the doubled value
     * @param value
     *            <code>byte</code> The value to be tested for double occurence
     */
    private void checkForMistake(final Element el, final byte value) {
        final int row = el.getRow();
        final int col = el.getCol();
        if (mistakeHint) {
            final int mistake = engine.check(row, col, value);
            if (mistake > -1) {
                this.highlightMistake(row, col, el.getQuad(), mistake, value);
                this.handleMistake(el);
            } else {
                // remove hint numbers
                this.manageHints(row, col, el.getQuad(), value);
            }
        }
    }

    /**
     * Coordinates the number handling of an element.<br>
     * The input values are filtered for numbers or empty Strings. The number of
     * empty cells are calculated and displayed on the statusBar bar. The input
     * is checked for sudoku faults and sounds are played for success or number
     * input if selected in the options dialog. After finishing Sudoku a dialog
     * is displayed.
     * 
     * @param el
     *            <code>Element</code>
     */
    public void handleInput(final Element el) {
        final String s = el.getFieldText().trim();
        final int row = el.getRow();
        final int col = el.getCol();
        // nur ein- oder mehrstellige Zahlen zulassen
        if (!s.matches("[1-9]") && !s.equals("")) {
            System.err.println("Nur Zahlen zwischen 1 und 9 sind erlaubt!");
            JOptionPane
                    .showMessageDialog(this, "Bitte nur Zahlen zwischen 1 und "
                            + ELEMENTS_IN_ROW
                            + " eingeben!", "Nur Zahlen sind erlaubt!", JOptionPane.OK_OPTION, icon);
            this.handleMistake(el);
            return;
        }

        // Show number of empty elements
        statusBar.setFilledOutCellsLabelText(this
                .getFilledCellsCount(), ELEMENTS_IN_ROW);
        // Abbruch bei leerer Zelle
        if (!s.equals("")) {
            final Byte b = Byte.valueOf(s).byteValue();
            // Update the array and highlight equal numbers.
            engine.setValueToStartArrayPos(row, col, b);
            this.highlightNumber(s);
            // if the user wants to get a hint in case of a fault
            this.checkForMistake(el, b);
        } else {
            // if a number is deleted remove it from the array and return
            // without playing any sound.
            engine.setValueToStartArrayPos(row, col, (byte) 0);
            return;
        }
        this.showSuccess(s);
    }

    /**
     * Enables cursor navigation by arrow keys
     * 
     * @param e
     *            the element triggering the key event
     * @param keyCode
     *            code of the key triggered by the given element
     */
    public void walkCursor(Element e, int keyCode) {
        Element elem = null;
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                elem = elements[i][j];
                if (elem.equals(e)) {
                    int row = elem.getRow();
                    int col = elem.getCol();
                    int destRow = row;
                    int destCol = col;

                    if (keyCode == KeyEvent.VK_DOWN) {
                        destRow = row < ELEMENTS_IN_ROW - 1 ? row + 1 : 0;
                    } else if (keyCode == KeyEvent.VK_UP) {
                        destRow = row > 0 ? row - 1 : ELEMENTS_IN_ROW - 1;
                    } else if (keyCode == KeyEvent.VK_LEFT) {
                        destCol = col > 0 ? col - 1 : ELEMENTS_IN_ROW - 1;
                    } else if (keyCode == KeyEvent.VK_RIGHT) {
                        destCol = col < ELEMENTS_IN_ROW - 1 ? col + 1 : 0;
                    }
                    elements[destRow][destCol].requestFocus();
                    this.highlightNumber(elements[destRow][destCol]
                            .getFieldText());
                }
            }
        }
    }

    /**
     * Clears all hint labels
     *
     */
    private void clearHintLabels() {
        for (int i = 0; i < elements.length; ++i) {
            for (int j = 0; j < elements[i].length; ++j) {
                elements[i][j].clearHintLabelText();
            }
        }
    }

    /**
     * If a digit already exists in an element's hint text remove it.
     * 
     * @param row
     *            <code>int</code> the row
     * @param col
     *            <code>int</code> the column
     * @param quad
     *            <code>int</code> the square
     * @param value
     *            <code>int</code> the digit
     */
    private void manageHints(final int row, final int col, final int quad,
            final int value) {
        String txt, intVal;
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                txt = elements[i][j].getLabel().getText();
                if (elements[i][j].getCol() == col
                        || elements[i][j].getRow() == row
                        || elements[i][j].getQuad() == quad) {
                    intVal = Integer.valueOf(value).toString();
                    if (txt.contains(intVal)) {
                        txt = txt.replaceAll(intVal, "");
                        elements[i][j].getLabel().setText(txt);
                    }
                }
            }
        }
    }

    /****************************** Color Handling ***************************/

    /**
     * Searches for all occurences of <code>number</code> and highlights its
     * background.
     * 
     * @param number
     *            <code>String</code>, the number of the current cell.
     */
    public void highlightNumber(final String number) {
        // reset background-color
        this.setElementsDefaultColors();
        for (int i = 0; i < elements.length; i++) {
            for (int j = 0; j < elements[i].length; j++) {
                if (!number.equals("")
                        && elements[i][j].getFieldText().equals(number)) {
                    elements[i][j]
                            .setBackground(HIGHLIGHT_BG_COLOR, HIGHLIGHT_BG_COLOR);
                }
                elements[i][j]
                        .setSelectionColor(elements[i][j].getBackground());
            }
        }
    }

    /**
     * Switches the element's view from number view to comment view and vice
     * versa.
     */
    public void switchView() {
        HintLabel label;
        NumberTextField field;
        for (int i = 0; i < elements.length; ++i) {
            for (int j = 0; j < elements[i].length; ++j) {
                label = elements[i][j].getLabel();
                field = elements[i][j].getField();
                // don't switch cells with numbers in fields
                if (!field.getText().equals("")) {
                    continue;
                }
                // if there are hints make the field transparent
                if (label.getText().matches("<html>[1-9]+</html>")) {
                    label.setOpaque(true);
                    field.setOpaque(false);
                }
                // switch the Z-orders of label and field
                if (elements[i][j].getComponentZOrder(label) == 0) {
                    elements[i][j].setComponentZOrder(label, 1);
                    label.setBorder(null);
                    elements[i][j].setComponentZOrder(field, 0);
                } else if (elements[i][j].getComponentZOrder(label) == 1) {
                    elements[i][j].setComponentZOrder(label, 0);
                    elements[i][j].setComponentZOrder(field, 1);
                    label.setBorder(HintLabel.ACTIVE_BORDER);
                }
                label.setVisible(field.isVisible());
                field.setVisible(label.isVisible());
                elements[i][j].repaint();
                elements[i][j].revalidate();
            }
        }
        this.setElementsDefaultColors();
    }

    /**
     * Controlls the alternation of light and dark of the square's background.
     * 
     * @see #switchContrastColors(boolean)
     */
    public void setElementsDefaultColors() {
        if (elements == null) {
            return;
        }
        for (int i = 0; i < ELEMENTS_IN_ROW; i++) {
            for (int j = 0; j < ELEMENTS_IN_ROW; j++) {
                final int quad = i / SQUARE_ROW_ELEMENTS * SQUARE_ROW_ELEMENTS
                        + j / SQUARE_ROW_ELEMENTS;

                if (quad % 2 == 0) {
                    elements[i][j].setForeground(lightFgColor, lightFgColor);
                    elements[i][j].setBackground(lightBgColor, lightBgColor);
                    elements[i][j].setBorderColor(borderColorDark);
                } else {
                    elements[i][j].setForeground(darkFgColor, darkFgColor);
                    elements[i][j].setBackground(darkBgColor, darkBgColor);
                    elements[i][j].setBorderColor(borderColorLight);
                }
            }
        }
    }

    /**
     * Changes the background and foreground color depending on the state of the
     * contrast mode.
     * 
     * @param selected
     *            <code>boolean</code>, <code>true</code> will switch to
     *            contrast mode. The background is set to black and white.
     *            Mistake's highlighting will be red. If <code>selected</code>
     *            is <code>false</code> the background and foreground colors are
     *            set to default.
     */
    private void switchContrastColors(final boolean selected) {
        contrast = selected;
        lightFgColor = selected ? FG_COLOR_DARK_ACCESS : FG_COLOR;
        darkFgColor = selected ? FG_COLOR_LIGHT_ACCESS : FG_COLOR;
        lightBgColor = selected ? BG_COLOR_LIGHT_ACCESS : BG_COLOR_LIGHT;
        darkBgColor = selected ? BG_COLOR_DARK_ACCESS : BG_COLOR_DARK;
        borderColorLight = selected ? BORDER_COLOR_LIGHT_ACCESS : BORDER_COLOR;
        borderColorDark = selected ? BORDER_COLOR_DARK_ACCESS : BORDER_COLOR;
    }
}

/**
 * The class manages sudoku's mistake timing.<br>
 * If a mistake occures the belonging line, row and square are highlighted and
 * the wrong number is deleted after a decent time that is defined by
 * {@link de.yourwebs.sudoku.gui.SudokuGUI#mistakeDelay}
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
class MistakeTimer extends TimerTask {
    /**
     * Reference to the main frame
     */
    SudokuGUI gui;
    /**
     * The element in which the mistake occures
     */
    Element el;

    /**
     * Creates a timer task that manages the occurence of a mistake displayed by
     * the given element
     * 
     * @param gui
     *            the game's panel
     * @param el
     *            the element displaying the mistake
     */
    public MistakeTimer(final SudokuGUI gui, final Element el) {
        this.gui = gui;
        this.el = el;
    }

    /**
     * Clears the element's text and switches foreground and background colors
     * to default
     */
    @Override
    public void run() {
        if (el.getFieldText().equals("")) {
            return;
        }
        el.setFieldText("");
        gui.setElementsDefaultColors();
    }
}

/**
 * Used for resizing the game's gui
 * 
 * @author J&ouml;rg Czeschla, <a href="https://javabeginners.de" target=
 *         "_blank">javabeginners.de</a>
 *
 */
class SudokuComponentAdapter extends ComponentAdapter {

    /**
     * The main frame
     */
    SudokuGUI gui;

    /**
     * Creates a component listener, that adjusts the main frame's components
     * while it's resized
     * 
     * @param gui
     *            the main frame
     */
    public SudokuComponentAdapter(final SudokuGUI gui) {
        this.gui = gui;
    }

    @Override
    public void componentResized(final ComponentEvent e) {
        final int faktor = ((JFrame) e.getSource()).getSize().width
                / SudokuGUI.SQUARE_ROW_ELEMENTS;
        gui.resizeGUI(faktor);
    }
}
